@@ -3,7 +3,7 @@ BEGIN {
   $POE::Quickie::AUTHORITY = 'cpan:HINRIK';
 }
 BEGIN {
-  $POE::Quickie::VERSION = '0.14';
+  $POE::Quickie::VERSION = '0.15';
 }
 
 use strict;
@@ -101,17 +101,22 @@ sub run {
     # propagate possible exception from POE::Wheel::Run->new()
     croak $exception if $exception;
 
-    $self->{lazy}{$wheel->PID} = { } if defined $args{ResultEvent};
     return $wheel->PID;
 }
 
 sub _create_wheel {
     my ($kernel, $self, $args) = @_[KERNEL, OBJECT, ARG0];
 
-    my $program = $args->{Program};
-    if ($args->{AltFork}) {
+    my %data;
+    for my $arg (qw(AltFork Timeout Input Program Context ProgramArgs
+            StdoutEvent StderrEvent ExitEvent ResultEvent Tee Merged)) {
+        next if !exists $args->{$arg};
+        $data{$arg} = delete $args->{$arg};
+    }
+
+    if ($data{AltFork}) {
         my @inc = map { +'-I' => $_ } @INC;
-        $program = [$^X, @inc, '-e', $program];
+        $data{Program} = [$^X, @inc, '-e', $data{Program}];
     }
 
     my $wheel;
@@ -121,19 +126,16 @@ sub _create_wheel {
             StdinEvent  => '_child_stdin',
             StdoutEvent => '_child_stdout',
             StderrEvent => '_child_stderr',
-            Program     => $program,
-            (defined $args->{ProgramArgs}
-                ? (ProgramArgs => $args->{ProgramArgs})
+            Program     => $data{Program},
+            (defined $data{ProgramArgs}
+                ? (ProgramArgs => $data{ProgramArgs})
                 : ()
             ),
             ($^O ne 'Win32'
                 ? (CloseOnCall => 1)
                 : ()
             ),
-            (defined $args->{WheelArgs}
-                ? (%{ $args->{WheelArgs} })
-                : ()
-            ),
+            %$args,
         );
     };
 
@@ -142,17 +144,20 @@ sub _create_wheel {
         return $@;
     }
 
-    $self->{wheels}{$wheel->ID}{obj} = $wheel;
-    $self->{wheels}{$wheel->ID}{args} = $args;
-    $self->{wheels}{$wheel->ID}{alive} = 2;
+    $data{obj} = $wheel;
+    $data{extra_args} = $args;
+    $self->{wheels}{$wheel->ID} = \%data;
 
-    if (defined $args->{Input}) {
-        $wheel->put($args->{Input});
+    if (defined $data{Input}) {
+        $wheel->put($data{Input});
+    }
+    else {
+        $wheel->shutdown_stdin();
     }
 
-    if (defined $args->{Timeout}) {
-        $self->{wheels}{$wheel->ID}{alrm}
-            = $kernel->delay_set('_child_timeout', $args->{Timeout}, $wheel->ID);
+    if ($data{Timeout}) {
+        $data{alrm} =
+            $kernel->delay_set('_child_timeout', $data{Timeout}, $wheel->ID);
     }
 
     $kernel->sig_child($wheel->PID, '_child_signal');
@@ -174,49 +179,51 @@ sub _child_signal {
     my $id = $self->_pid_to_id($pid);
 
     my $data = $self->{wheels}{$id};
-    $data->{status} = $status;
 
     my $s = $status >> 8;
-    if ($s != 0 && !exists $data->{args}{ExitEvent}
+    if ($s != 0 && !exists $data->{ExitEvent}
             && !exists $data->{ResultEvent}) {
         warn "Child $pid exited with nonzero status $s\n";
     }
 
-    $kernel->alarm_remove($data->{alrm});
+    $kernel->alarm_remove($data->{alrm}) if $data->{Timeout};
+    if ($data->{lazy}) {
+        $self->{lazy}{$id} = {
+            merged => $data->{merged},
+            stdout => $data->{stdout},
+            stderr => $data->{stderr},
+            status => $status,
+        }
+    }
+    delete $self->{wheels}{$id};
 
-    if (defined $data->{args}{ExitEvent}) {
+    if (defined $data->{ExitEvent}) {
         $kernel->call(
             $self->{parent_id},
-            $data->{args}{ExitEvent},
+            $data->{ExitEvent},
             $status,
             $pid,
-            (defined $data->{args}{Context}
-                ? $data->{args}{Context}
+            (defined $data->{Context}
+                ? $data->{Context}
                 : ()),
         );
     }
 
-    if ($self->{lazy}{$pid}) {
-        $self->{lazy}{$pid}{results} = $self->_lazy_collate_results($pid);
-
+    if (defined $data->{ResultEvent}) {
+        $kernel->call(
+            $self->{parent_id},
+            $data->{ResultEvent},
+            $pid,
+            $data->{stdout},
+            $data->{stderr},
+            $data->{merged},
+            $status,
+            (defined $data->{Context}
+                ? $data->{Context}
+                : ()),
+        );
     }
-    delete $self->{wheels}{$id};
-    return if !defined $data->{args}{ResultEvent};
 
-    my ($stdout, $stderr, $merged) = @{ $self->{lazy}{$pid}{results} };
-    delete $self->{lazy}{$pid};
-    $kernel->call(
-        $self->{parent_id},
-        $data->{args}{ResultEvent},
-        $pid,
-        $stdout,
-        $stderr,
-        $merged,
-        $status,
-        (defined $data->{args}{Context}
-            ? $data->{args}{Context}
-            : ()),
-    );
     return;
 }
 
@@ -235,26 +242,26 @@ sub _child_stdin {
 sub _child_stdout {
     my ($kernel, $self, $output, $id) = @_[KERNEL, OBJECT, ARG0, ARG1];
 
-    my $pid = $self->{wheels}{$id}{obj}->PID;
+    my $data = $self->{wheels}{$id};
 
-    if ($self->{lazy}{$pid}) {
-        push @{ $self->{lazy}{$pid}{merged} }, $output;
-        push @{ $self->{lazy}{$pid}{stdout} }, $output;
+    if ($data->{lazy} || defined $data->{ResultEvent}) {
+        push @{ $data->{merged} }, $output;
+        push @{ $data->{stdout} }, $output;
 
-        if ($self->{lazy}{$pid}{Tee}) {
+        if ($data->{lazy}{Tee}) {
             print $output, "\n";
         }
     }
-    elsif (!exists $self->{wheels}{$id}{args}{StdoutEvent}) {
+    elsif (!exists $data->{StdoutEvent}) {
         print "$output\n";
     }
-    elsif (defined (my $event = $self->{wheels}{$id}{args}{StdoutEvent})) {
-        my $context = $self->{wheels}{$id}{args}{Context};
+    elsif (defined (my $event = $data->{StdoutEvent})) {
+        my $context = $data->{Context};
         $kernel->call(
             $self->{parent_id},
             $event,
             $output,
-            $pid,
+            $data->{obj}->PID,
             (defined $context ? $context : ()),
         );
     }
@@ -265,28 +272,28 @@ sub _child_stdout {
 sub _child_stderr {
     my ($kernel, $self, $error, $id) = @_[KERNEL, OBJECT, ARG0, ARG1];
 
-    my $pid = $self->{wheels}{$id}{obj}->PID;
+    my $data = $self->{wheels}{$id};
 
-    if ($self->{lazy}{$pid}) {
-        push @{ $self->{lazy}{$pid}{merged} }, $error;
-        push @{ $self->{lazy}{$pid}{stderr} }, $error;
+    if ($data->{lazy} || defined $data->{ResultEvent}) {
+        push @{ $data->{merged} }, $error;
+        push @{ $data->{stderr} }, $error;
 
-        if ($self->{lazy}{$pid}{Tee}) {
-            $self->{lazy}{$pid}{Merged}
+        if ($data->{lazy}{Tee}) {
+            $data->{lazy}{Merged}
                 ? print $error, "\n"
                 : warn $error, "\n";
         }
     }
-    elsif (!exists $self->{wheels}{$id}{args}{StderrEvent}) {
+    elsif (!exists $data->{StderrEvent}) {
         warn "$error\n";
     }
-    elsif (defined (my $event = $self->{wheels}{$id}{args}{StderrEvent})) {
-        my $context = $self->{wheels}{$id}{args}{Context};
+    elsif (defined (my $event = $data->{StderrEvent})) {
+        my $context = $data->{Context};
         $kernel->call(
             $self->{parent_id},
             $event,
             $error,
-            $pid,
+            $data->{obj}->PID,
             (defined $context ? $context : ()),
         );
     }
@@ -330,7 +337,7 @@ sub processes {
     my %wheels;
     for my $id (keys %{ $self->{wheels} }) {
         my $pid = $self->{wheels}{$id}{obj}->PID;
-        $wheels{$pid} = $self->{wheels}{$id}{args}{Context};
+        $wheels{$pid} = $self->{wheels}{$id}{Context};
     }
 
     return \%wheels;
@@ -356,33 +363,19 @@ sub _lazy_run {
     );
 
     my $id = $self->_pid_to_id($pid);
-    $self->{lazy}{$pid} = { %args };
+    $self->{wheels}{$id}{lazy} = {
+        Tee    => $args{Tee},
+        Merged => $args{Merged},
+    };
 
     my $parent_id = $poe_kernel->get_active_session->ID;
     $poe_kernel->refcount_increment($parent_id, __PACKAGE__);
-    $poe_kernel->refcount_increment($self->{session_id}, __PACKAGE__);
     $poe_kernel->run_one_timeslice() while $self->{wheels}{$id};
     $poe_kernel->refcount_decrement($parent_id, __PACKAGE__);
-    $poe_kernel->refcount_decrement($self->{session_id}, __PACKAGE__);
 
-    my ($stdout, $stderr, $merged, $status) = @{ $self->{lazy}{$pid}{results} };
-    delete $self->{lazy}{$pid};
-
-    return $merged, $status if $args{Merged};
-    return $stdout, $stderr, $status;
-}
-
-sub _lazy_collate_results {
-    my ($self, $pid) = @_;
-
-    my $data   = $self->{lazy}{$pid};
-    my $id     = $self->_pid_to_id($pid);
-    my $status = $self->{wheels}{$id}{status};
-    my $stdout = join '', map { "$_\n" } @{ $data->{stdout} || [] };
-    my $stderr = join '', map { "$_\n" } @{ $data->{stderr} || [] };
-    my $merged = join '', map { "$_\n" } @{ $data->{merged} || [] };
-
-    return [$stdout, $stderr, $merged, $status];
+    my $data = delete $self->{lazy}{$id};
+    return $data->{merged}, $data->{status} if $args{Merged};
+    return $data->{stdout}, $data->{stderr}, $data->{status};
 }
 
 sub quickie_run {
@@ -471,7 +464,8 @@ POE::Quickie - A lazy way to wrap blocking code and programs
  # only called when the task is finished
  sub result {
      my ($pid, $stdout, $stderr, $merged, $status, $context) = @_[ARG0..$#_];
-     print "got all this output in the context of '$context': '$output'\n";
+     print "got all this output in the context of '$context':\n";
+     print "$_\n" for @$stdout;
  }
 
 =head1 DESCRIPTION
@@ -556,9 +550,9 @@ must be a string) as the code argument (L<I<-e>|perlrun>), and the current
 L<C<@INC>|perlvar> passed as include arguments (L<I<-I>|perlrun>). Default
 is false.
 
-B<'WheelArgs'> (optional), a hash reference of options which will be passed
-verbatim to the underlying POE::Wheel::Run object's constructor. Possibly
-useful if you want to change the input/output filters and such.
+All other arguments will be passed to POE::Wheel::Run's
+L<C<new>|POE::Wheel::Run/new> method. Useful if you want to specify the
+input/output filters and such.
 
 =head2 C<killall>
 
@@ -616,11 +610,11 @@ to the options to L<C<run>|/run>.
 
 =item C<ARG0>: the process id of the child process
 
-=item C<ARG1>: all the stdout generated by the child process
+=item C<ARG1>: an array of every chunk of stdout generated by the child process
 
-=item C<ARG2>: all the stderr generated by the child process
+=item C<ARG2>: an array of every chunk of stderr generated by the child process
 
-=item C<ARG3>: merged stdout/stderr generated by the child process
+=item C<ARG3>: an array of the interleaved stdout and stderr chunks
 
 =item C<ARG4>: the exit code (L<C<$?>|perlvar>) of the child process
 
@@ -659,22 +653,24 @@ child process.
 
 =head3 C<quickie_tee>
 
-Returns 3 values: the stdout, stderr, and exit code (L<C<$?>|perlvar>) of the
-child process. In addition, it will echo the stdout/stderr to your process'
-stdout/stderr. Beware that stdout and stderr in the merged result are not
-guaranteed to be properly ordered due to buffering.
+Returns 3 values: an array of all stdout chunks, an array of all stderr
+chunks, and the exit code (L<C<$?>|perlvar>) of the child process. In addition,
+it will echo the stdout/stderr to your process' stdout/stderr.
 
 =head3 C<quickie_merged>
 
-Returns 2 values: the merged stdout & stderr, and exit code (L<C<$?>|perlvar>)
-of the child process.
+Returns 2 values: an array of interleaved stdout and stderr chunks, and the
+exit code (L<C<$?>|perlvar>) of the child process. Beware that stdout and
+stderr in the merged result are not guaranteed to be properly ordered due to
+buffering.
 
 =head3 C<quickie_tee_merged>
 
-Returns 2 values: the merged stdout & stderr, and exit code (L<C<$?>|perlvar>)
-of the child process. In addition, it will echo the merged stdout & stderr to
-your process' stdout. Beware that stdout and stderr in the merged result are
-not guaranteed to be properly ordered due to buffering.
+Returns 2 values: an array of interleaved stdout and stderr chunks, and the
+exit code (L<C<$?>|perlvar>) of the child process. In addition, it will echo
+the merged stdout & stderr to your process' stdout. Beware that stdout and
+stderr in the merged result are not guaranteed to be properly ordered due to
+buffering.
 
 =head1 AUTHOR
 
